"use strict";
Object.defineProperty(exports, "__esModule", { value: !0 });
var jsxRuntime = require("react/jsx-runtime"), react = require("react"), visualEditingCsm = require("@sanity/visual-editing-csm"), csm = require("@sanity/client/csm");
const htmlElements = [
  "a",
  "abbr",
  "address",
  "area",
  "article",
  "aside",
  "audio",
  "b",
  "base",
  "bdi",
  "bdo",
  "big",
  "blockquote",
  "body",
  "br",
  "button",
  "canvas",
  "caption",
  "cite",
  "code",
  "col",
  "colgroup",
  "data",
  "datalist",
  "dd",
  "del",
  "details",
  "dfn",
  "dialog",
  "div",
  "dl",
  "dt",
  "em",
  "embed",
  "fieldset",
  "figcaption",
  "figure",
  "footer",
  "form",
  "h1",
  "h2",
  "h3",
  "h4",
  "h5",
  "h6",
  "head",
  "header",
  "hgroup",
  "hr",
  "html",
  "i",
  "iframe",
  "img",
  "input",
  "ins",
  "kbd",
  "keygen",
  "label",
  "legend",
  "li",
  "link",
  "main",
  "map",
  "mark",
  "menu",
  "menuitem",
  "meta",
  "meter",
  "nav",
  "noscript",
  "object",
  "ol",
  "optgroup",
  "option",
  "output",
  "p",
  "param",
  "picture",
  "pre",
  "progress",
  "q",
  "rp",
  "rt",
  "ruby",
  "s",
  "samp",
  "script",
  "section",
  "select",
  "small",
  "source",
  "span",
  "strong",
  "style",
  "sub",
  "summary",
  "sup",
  "table",
  "tbody",
  "td",
  "textarea",
  "tfoot",
  "th",
  "thead",
  "time",
  "title",
  "tr",
  "track",
  "u",
  "ul",
  "var",
  "video",
  "wbr",
  "webview"
], SanityElement = react.forwardRef(function(props, ref) {
  const { as: As, children: node, ...restProps } = props;
  return node?.source ? (
    // @ts-expect-error @TODO find out why children is not allowed in IntrinsicAttributes
    /* @__PURE__ */ jsxRuntime.jsx(As, { ...restProps, "data-sanity": visualEditingCsm.encodeSanityNodeData(node.source), ref, children: node.value })
  ) : /* @__PURE__ */ jsxRuntime.jsx(As, { ...restProps, children: node?.value });
}), svgElements = [
  "animate",
  "circle",
  "defs",
  "desc",
  "ellipse",
  "g",
  "image",
  "line",
  "filter",
  "marker",
  "mask",
  "metadata",
  "path",
  "pattern",
  "polygon",
  "polyline",
  "rect",
  "stop",
  "svg",
  "switch",
  "symbol",
  "text",
  "tspan",
  "use",
  "view",
  "clipPath",
  "feBlend",
  "feColorMatrix",
  "feComponentTransfer",
  "feComposite",
  "feConvolveMatrix",
  "feDiffuseLighting",
  "feDisplacementMap",
  "feDistantLight",
  "feDropShadow",
  "feFlood",
  "feFuncA",
  "feFuncB",
  "feFuncG",
  "feFuncR",
  "feGaussianBlur",
  "feImage",
  "feMerge",
  "feMergeNode",
  "feMorphology",
  "feOffset",
  "fePointLight",
  "feSpecularLighting",
  "feSpotLight",
  "feTile",
  "feTurbulence",
  "foreignObject",
  "linearGradient",
  "radialGradient",
  "textPath"
];
function isHTMLElement(name) {
  return htmlElements.includes(name);
}
function isSVGElement(name) {
  return svgElements.includes(name);
}
const sanity = new Proxy({}, {
  get(target, prop) {
    if (isHTMLElement(prop)) {
      if (target[prop]) return target[prop];
      const SanityComponent = react.forwardRef(function(props, ref) {
        return /* @__PURE__ */ jsxRuntime.jsx(SanityElement, { ...props, as: prop, ref });
      });
      return SanityComponent.displayName = `sanity.${prop}`, target[prop] = SanityComponent, SanityComponent;
    }
    if (isSVGElement(prop)) {
      if (target[prop]) return target[prop];
      const SanityComponent = react.forwardRef(function(props, ref) {
        return /* @__PURE__ */ jsxRuntime.jsx(SanityElement, { ...props, as: prop, ref });
      });
      return SanityComponent.displayName = `sanity.${prop}`, target[prop] = SanityComponent, SanityComponent;
    }
    throw new Error(`No such element: ${prop}`);
  }
}), SANITY_KEYS = [
  "_createdAt",
  "_dataset",
  "_id",
  "_key",
  "_originalId",
  "_projectId",
  "_ref",
  "_rev",
  "_strengthenOnPublish",
  "_type",
  "_updatedAt",
  "_weak"
];
function isRecord(value) {
  return typeof value == "object" && value !== null && !Array.isArray(value);
}
function isArray(value) {
  return value !== null && Array.isArray(value);
}
function isSourceNode(t) {
  return isRecord(t) && t.$$type$$ === "sanity";
}
function unwrapData(wrapper) {
  if (wrapper !== void 0) {
    if (wrapper === null)
      return null;
    if (isSourceNode(wrapper))
      return wrapper.value;
    if (isArray(wrapper))
      return wrapper.map((item) => unwrapData(item));
    if (isRecord(wrapper))
      return Object.fromEntries(
        Object.entries(wrapper).map(
          ([k, v]) => SANITY_KEYS.includes(k) ? [k, v] : [k, unwrapData(v)]
        )
      );
    throw new Error("invalid wrapped value");
  }
}
function resolveSanityNode(context, csm$1, resultPath, keyedResultPath) {
  const { mapping, pathSuffix } = csm.resolveMapping(resultPath, csm$1) || {};
  if (!mapping || mapping.source.type === "literal" || mapping.source.type === "unknown")
    return;
  const sourceDoc = csm$1.documents[mapping.source.document], sourceBasePath = csm$1.paths[mapping.source.path];
  if (sourceDoc && sourceBasePath)
    return {
      baseUrl: context.baseUrl,
      workspace: context.workspace,
      tool: context.tool,
      type: sourceDoc._type,
      id: csm.getPublishedId(sourceDoc._id),
      path: csm.studioPath.toString(
        csm.jsonPathToStudioPath(
          csm.resolvedKeyedSourcePath({
            keyedResultPath,
            pathSuffix,
            sourceBasePath
          })
        )
      )
    };
}
function wrapData(context, value, sourceMap, resultPath = [], keyedResultPath = []) {
  if (value !== void 0)
    return value === null ? null : isArray(value) ? value.map(
      (t, idx) => wrapData(
        context,
        t,
        sourceMap,
        resultPath.concat(idx),
        keyedResultPath.concat(
          isRecord(t) && "_key" in t && typeof t._key == "string" ? { _key: t._key, _index: idx } : idx
        )
      )
    ) : isRecord(value) ? Object.fromEntries(
      Object.entries(value).map(
        ([k, v]) => SANITY_KEYS.includes(k) ? [k, v] : [k, wrapData(context, v, sourceMap, resultPath.concat(k), keyedResultPath.concat(k))]
      )
    ) : {
      $$type$$: "sanity",
      path: csm.studioPath.toString(csm.jsonPathToStudioPath(resultPath)) || void 0,
      source: sourceMap ? resolveSanityNode(context, sourceMap, resultPath, keyedResultPath) : void 0,
      value
    };
}
Object.defineProperty(exports, "encodeSanityNodeData", {
  enumerable: !0,
  get: function() {
    return visualEditingCsm.encodeSanityNodeData;
  }
});
exports.sanity = sanity;
exports.unwrapData = unwrapData;
exports.wrapData = wrapData;
//# sourceMappingURL=jsx.cjs.map
